@@ -11,8 +11,9 @@ sql_quantile <- function(f,
 
     sql <- switch(style,
       infix = sql_call2(f, x, probs),
-      ordered = build_sql(
-        sql_call2(f, probs), " WITHIN GROUP (ORDER BY ", x, ")"
+      ordered = glue_sql2(
+        sql_current_con(),
+        sql_call2(f, probs), " WITHIN GROUP (ORDER BY {x})"
       )
     )
 
@@ -35,10 +36,9 @@ sql_median <- function(f,
   }
 }
 
-check_probs <- function(probs) {
-  if (!is.numeric(probs)) {
-    cli_abort("{.arg probs} must be numeric")
-  }
+check_probs <- function(probs, call = caller_env()) {
+  # TODO min, max? Inf? NA?
+  check_number_decimal(probs, call = call)
 
   if (length(probs) > 1) {
     cli_abort("SQL translation only supports single value for {.arg probs}.")
