@@ -3,106 +3,142 @@
 test_that("custom scalar translated correctly", {
   local_con(simulate_mssql())
 
-  expect_equal(translate_sql(as.logical(x)),   sql("TRY_CAST(`x` AS BIT)"))
-  expect_equal(translate_sql(as.numeric(x)),   sql("TRY_CAST(`x` AS FLOAT)"))
-  expect_equal(translate_sql(as.integer(x)),   sql("TRY_CAST(TRY_CAST(`x` AS NUMERIC) AS INT)"))
-  expect_equal(translate_sql(as.integer64(x)), sql("TRY_CAST(TRY_CAST(`x` AS NUMERIC(38, 0)) AS BIGINT)"))
-  expect_equal(translate_sql(as.double(x)),    sql("TRY_CAST(`x` AS FLOAT)"))
-  expect_equal(translate_sql(as.character(x)), sql("TRY_CAST(`x` AS VARCHAR(MAX))"))
-  expect_equal(translate_sql(log(x)),          sql("LOG(`x`)"))
-  expect_equal(translate_sql(nchar(x)),        sql("LEN(`x`)"))
-  expect_equal(translate_sql(atan2(x)),        sql("ATN2(`x`)"))
-  expect_equal(translate_sql(ceiling(x)),      sql("CEILING(`x`)"))
-  expect_equal(translate_sql(ceil(x)),         sql("CEILING(`x`)"))
-  expect_equal(translate_sql(substr(x, 1, 2)), sql("SUBSTRING(`x`, 1, 2)"))
-  expect_equal(translate_sql(trimws(x)),       sql("LTRIM(RTRIM(`x`))"))
-  expect_equal(translate_sql(paste(x, y)),     sql("`x` + ' ' + `y`"))
+  expect_equal(test_translate_sql(as.logical(x)),   sql("TRY_CAST(`x` AS BIT)"))
+  expect_equal(test_translate_sql(as.numeric(x)),   sql("TRY_CAST(`x` AS FLOAT)"))
+  expect_equal(test_translate_sql(as.integer(x)),   sql("TRY_CAST(TRY_CAST(`x` AS NUMERIC) AS INT)"))
+  expect_equal(test_translate_sql(as.integer64(x)), sql("TRY_CAST(TRY_CAST(`x` AS NUMERIC(38, 0)) AS BIGINT)"))
+  expect_equal(test_translate_sql(as.double(x)),    sql("TRY_CAST(`x` AS FLOAT)"))
+  expect_equal(test_translate_sql(as.character(x)), sql("TRY_CAST(`x` AS VARCHAR(MAX))"))
+  expect_equal(test_translate_sql(log(x)),          sql("LOG(`x`)"))
+  expect_equal(test_translate_sql(nchar(x)),        sql("LEN(`x`)"))
+  expect_equal(test_translate_sql(atan2(x)),        sql("ATN2(`x`)"))
+  expect_equal(test_translate_sql(ceiling(x)),      sql("CEILING(`x`)"))
+  expect_equal(test_translate_sql(ceil(x)),         sql("CEILING(`x`)"))
+  expect_equal(test_translate_sql(substr(x, 1, 2)), sql("SUBSTRING(`x`, 1, 2)"))
+  expect_equal(test_translate_sql(trimws(x)),       sql("LTRIM(RTRIM(`x`))"))
+  expect_equal(test_translate_sql(paste(x, y)),     sql("`x` + ' ' + `y`"))
   expect_equal(
-    translate_sql(if_else(x, "true", "false", "missing")),
+    test_translate_sql(if_else(x, "true", "false", "missing")),
     sql("CASE WHEN `x` THEN 'true' WHEN NOT `x` THEN 'false' WHEN (`x` IS NULL) THEN 'missing' END")
   )
   expect_equal(
-    translate_sql(ifelse(x, "true", "false")),
+    test_translate_sql(ifelse(x, "true", "false")),
     sql("IIF(`x`, 'true', 'false')")
   )
   expect_equal(
-    translate_sql(ifelse(x, "true", NULL)),
+    test_translate_sql(ifelse(x, "true", NULL)),
     sql("IIF(`x`, 'true', NULL)")
   )
   expect_equal(
-    translate_sql(if(x) "true" else "false"),
+    test_translate_sql(if(x) "true" else "false"),
     sql("IIF(`x`, 'true', 'false')")
   )
 
-  expect_error(translate_sql(bitwShiftL(x, 2L)), sql("not available"))
-  expect_error(translate_sql(bitwShiftR(x, 2L)), sql("not available"))
+  expect_error(test_translate_sql(bitwShiftL(x, 2L)), sql("not available"))
+  expect_error(test_translate_sql(bitwShiftR(x, 2L)), sql("not available"))
 })
 
 test_that("contents of [ have bool context", {
   local_con(simulate_mssql())
   local_context(list(clause = "SELECT"))
 
-  expect_equal(translate_sql(x[x > y]), sql("CASE WHEN (`x` > `y`) THEN (`x`) END"))
+  expect_equal(test_translate_sql(x[x > y]), sql("CASE WHEN (`x` > `y`) THEN (`x`) END"))
 })
 
 test_that("custom stringr functions translated correctly", {
   local_con(simulate_mssql())
 
-  expect_equal(translate_sql(str_length(x)),     sql("LEN(`x`)"))
+  expect_equal(test_translate_sql(str_length(x)),     sql("LEN(`x`)"))
 })
 
 test_that("custom aggregators translated correctly", {
   local_con(simulate_mssql())
 
-  expect_equal(translate_sql(sd(x, na.rm = TRUE), window = FALSE),  sql("STDEV(`x`)"))
-  expect_equal(translate_sql(var(x, na.rm = TRUE), window = FALSE), sql("VAR(`x`)"))
+  expect_equal(test_translate_sql(sd(x, na.rm = TRUE), window = FALSE),  sql("STDEV(`x`)"))
+  expect_equal(test_translate_sql(var(x, na.rm = TRUE), window = FALSE), sql("VAR(`x`)"))
 
-  expect_error(translate_sql(cor(x), window = FALSE), "not available")
-  expect_error(translate_sql(cov(x), window = FALSE), "not available")
+  expect_error(test_translate_sql(cor(x), window = FALSE), "not available")
+  expect_error(test_translate_sql(cov(x), window = FALSE), "not available")
+
+  expect_equal(test_translate_sql(str_flatten(x), window = FALSE), sql("STRING_AGG(`x`, '')"))
+  expect_snapshot(error = TRUE, {
+    test_translate_sql(quantile(x, 0.5, na.rm = TRUE), window = FALSE)
+    test_translate_sql(median(x, na.rm = TRUE), window = FALSE)
+  })
 
-  expect_equal(translate_sql(str_flatten(x), window = FALSE), sql("STRING_AGG(`x`, '')"))
   expect_equal(
-    translate_sql(quantile(x, 0.5, na.rm = TRUE), window = FALSE),
-    sql("PERCENTILE_CONT(0.5) WITHIN GROUP (ORDER BY `x`) OVER ()")
+    test_translate_sql(all(x, na.rm = TRUE), window = FALSE),
+    sql("CAST(MIN(CAST(`x` AS INT)) AS BIT)")
   )
   expect_equal(
-    translate_sql(median(x, na.rm = TRUE), window = FALSE),
-    sql("PERCENTILE_CONT(0.5) WITHIN GROUP (ORDER BY `x`) OVER ()")
+    test_translate_sql(any(x, na.rm = TRUE), window = FALSE),
+    sql("CAST(MAX(CAST(`x` AS INT)) AS BIT)")
   )
 })
 
 test_that("custom window functions translated correctly", {
   local_con(simulate_mssql())
 
-  expect_equal(translate_sql(sd(x, na.rm = TRUE)),  sql("STDEV(`x`) OVER ()"))
-  expect_equal(translate_sql(var(x, na.rm = TRUE)), sql("VAR(`x`) OVER ()"))
+  expect_equal(test_translate_sql(sd(x, na.rm = TRUE)),  sql("STDEV(`x`) OVER ()"))
+  expect_equal(test_translate_sql(var(x, na.rm = TRUE)), sql("VAR(`x`) OVER ()"))
 
-  expect_equal(translate_sql(str_flatten(x)), sql("STRING_AGG(`x`, '') OVER ()"))
+  expect_equal(test_translate_sql(str_flatten(x)), sql("STRING_AGG(`x`, '') OVER ()"))
+
+  expect_equal(
+    test_translate_sql(quantile(x, 0.3, na.rm = TRUE), window = TRUE),
+    sql("PERCENTILE_CONT(0.3) WITHIN GROUP (ORDER BY `x`) OVER ()")
+  )
+  expect_equal(
+    test_translate_sql(median(x, na.rm = TRUE), window = TRUE),
+    sql("PERCENTILE_CONT(0.5) WITHIN GROUP (ORDER BY `x`) OVER ()")
+  )
+
+  expect_equal(
+    test_translate_sql(all(x, na.rm = TRUE)),
+    sql("CAST(MIN(CAST(`x` AS INT)) OVER () AS BIT)")
+  )
+  expect_equal(
+    test_translate_sql(any(x, na.rm = TRUE)),
+    sql("CAST(MAX(CAST(`x` AS INT)) OVER () AS BIT)")
+  )
 })
 
 test_that("custom lubridate functions translated correctly", {
   local_con(simulate_mssql())
-  expect_equal(translate_sql(as_date(x)),     sql("TRY_CAST(`x` AS DATE)"))
-  expect_equal(translate_sql(as_datetime(x)), sql("TRY_CAST(`x` AS DATETIME2)"))
-  expect_equal(translate_sql(today()),   sql("CAST(SYSDATETIME() AS DATE)"))
-  expect_equal(translate_sql(year(x)),   sql("DATEPART(YEAR, `x`)"))
-  expect_equal(translate_sql(day(x)),    sql("DATEPART(DAY, `x`)"))
-  expect_equal(translate_sql(mday(x)),   sql("DATEPART(DAY, `x`)"))
-  expect_equal(translate_sql(yday(x)),   sql("DATEPART(DAYOFYEAR, `x`)"))
-  expect_equal(translate_sql(hour(x)),   sql("DATEPART(HOUR, `x`)"))
-  expect_equal(translate_sql(minute(x)), sql("DATEPART(MINUTE, `x`)"))
-  expect_equal(translate_sql(second(x)), sql("DATEPART(SECOND, `x`)"))
-  expect_equal(translate_sql(month(x)), sql("DATEPART(MONTH, `x`)"))
-  expect_equal(translate_sql(month(x, label = TRUE, abbr = FALSE)), sql("DATENAME(MONTH, `x`)"))
-  expect_snapshot(error = TRUE, translate_sql(month(x, label = TRUE, abbr = TRUE)))
+  expect_equal(test_translate_sql(as_date(x)),     sql("TRY_CAST(`x` AS DATE)"))
+  expect_equal(test_translate_sql(as_datetime(x)), sql("TRY_CAST(`x` AS DATETIME2)"))
+  expect_equal(test_translate_sql(today()),   sql("CAST(SYSDATETIME() AS DATE)"))
+  expect_equal(test_translate_sql(year(x)),   sql("DATEPART(YEAR, `x`)"))
+  expect_equal(test_translate_sql(day(x)),    sql("DATEPART(DAY, `x`)"))
+  expect_equal(test_translate_sql(mday(x)),   sql("DATEPART(DAY, `x`)"))
+  expect_equal(test_translate_sql(yday(x)),   sql("DATEPART(DAYOFYEAR, `x`)"))
+  expect_equal(test_translate_sql(hour(x)),   sql("DATEPART(HOUR, `x`)"))
+  expect_equal(test_translate_sql(minute(x)), sql("DATEPART(MINUTE, `x`)"))
+  expect_equal(test_translate_sql(second(x)), sql("DATEPART(SECOND, `x`)"))
+  expect_equal(test_translate_sql(month(x)), sql("DATEPART(MONTH, `x`)"))
+  expect_equal(test_translate_sql(month(x, label = TRUE, abbr = FALSE)), sql("DATENAME(MONTH, `x`)"))
+  expect_snapshot(error = TRUE, test_translate_sql(month(x, label = TRUE, abbr = TRUE)))
 
-  expect_equal(translate_sql(quarter(x)), sql("DATEPART(QUARTER, `x`)"))
-  expect_equal(translate_sql(quarter(x, with_year = TRUE)), sql("(DATENAME(YEAR, `x`) + '.' + DATENAME(QUARTER, `x`))"))
-  expect_error(translate_sql(quarter(x, fiscal_start = 5)))
+  expect_equal(test_translate_sql(quarter(x)), sql("DATEPART(QUARTER, `x`)"))
+  expect_equal(test_translate_sql(quarter(x, with_year = TRUE)), sql("(DATENAME(YEAR, `x`) + '.' + DATENAME(QUARTER, `x`))"))
+  expect_error(test_translate_sql(quarter(x, fiscal_start = 5)))
 })
 
 test_that("last_value_sql() translated correctly", {
-  expect_equal(last_value_sql(simulate_mssql(), "x"), sql("LAST_VALUE(`x`) IGNORE NULLS"))
+  con <- simulate_mssql()
+  expect_equal(
+    translate_sql(last(x, na_rm = TRUE), vars_order = "a", con = con),
+    sql("LAST_VALUE(`x`) IGNORE NULLS OVER (ORDER BY `a` ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING)")
+  )
+})
+
+test_that("between translation respects context", {
+  local_con(simulate_mssql())
+
+  local_context(list(clause = "WHERE"))
+  expect_equal(test_translate_sql(between(a, 1L, 2L)), sql("`a` BETWEEN 1 AND 2"))
+  local_context(list(clause = "SELECT"))
+  expect_equal(test_translate_sql(between(a, 1L, 2L)), sql("IIF(`a` BETWEEN 1 AND 2, 1, 0)"))
 })
 
 # verb translation --------------------------------------------------------
@@ -181,18 +217,20 @@ test_that("generates custom sql", {
 })
 
 test_that("`sql_query_insert()` is correct", {
+  con <- simulate_mssql()
   df_y <- lazy_frame(
     a = 2:3, b = c(12L, 13L), c = -(2:3), d = c("y", "z"),
-    con = simulate_mssql(),
+    con = con,
     .name = "df_y"
   ) %>%
     mutate(c = c + 1)
 
   expect_snapshot(
     sql_query_insert(
-      con = simulate_mssql(),
-      x_name = ident("df_x"),
-      y = df_y,
+      con = con,
+      table = ident("df_x"),
+      from = sql_render(df_y, con, lvl = 1),
+      insert_cols = colnames(df_y),
       by = c("a", "b"),
       conflict = "ignore",
       returning_cols = c("a", b2 = "b")
@@ -201,36 +239,39 @@ test_that("`sql_query_insert()` is correct", {
 })
 
 test_that("`sql_query_append()` is correct", {
+  con <- simulate_mssql()
   df_y <- lazy_frame(
     a = 2:3, b = c(12L, 13L), c = -(2:3), d = c("y", "z"),
-    con = simulate_mssql(),
+    con = con,
     .name = "df_y"
   ) %>%
     mutate(c = c + 1)
 
   expect_snapshot(
     sql_query_append(
-      con = simulate_mssql(),
-      x_name = ident("df_x"),
-      y = df_y,
+      con = con,
+      table = ident("df_x"),
+      from = sql_render(df_y, con, lvl = 1),
+      insert_cols = colnames(df_y),
       returning_cols = c("a", b2 = "b")
     )
   )
 })
 
 test_that("`sql_query_update_from()` is correct", {
+  con <- simulate_mssql()
   df_y <- lazy_frame(
     a = 2:3, b = c(12L, 13L), c = -(2:3), d = c("y", "z"),
-    con = simulate_mssql(),
+    con = con,
     .name = "df_y"
   ) %>%
     mutate(c = c + 1)
 
   expect_snapshot(
     sql_query_update_from(
-      con = simulate_mssql(),
-      x_name = ident("df_x"),
-      y = df_y,
+      con = con,
+      table = ident("df_x"),
+      from = sql_render(df_y, con, lvl = 1),
       by = c("a", "b"),
       update_values = sql(
         c = "COALESCE(`df_x`.`c`, `...y`.`c`)",
@@ -252,8 +293,8 @@ test_that("`sql_query_delete()` is correct", {
   expect_snapshot(
     sql_query_delete(
       con = simulate_mssql(),
-      x_name = ident("df_x"),
-      y = df_y,
+      table = ident("df_x"),
+      from = sql_render(df_y, simulate_mssql(), lvl = 2),
       by = c("a", "b"),
       returning_cols = c("a", b2 = "b")
     )
@@ -261,18 +302,19 @@ test_that("`sql_query_delete()` is correct", {
 })
 
 test_that("`sql_query_upsert()` is correct", {
+  con <- simulate_mssql()
   df_y <- lazy_frame(
     a = 2:3, b = c(12L, 13L), c = -(2:3), d = c("y", "z"),
-    con = simulate_mssql(),
+    con = con,
     .name = "df_y"
   ) %>%
     mutate(c = c + 1)
 
   expect_snapshot(
     sql_query_upsert(
-      con = simulate_mssql(),
-      x_name = ident("df_x"),
-      y = df_y,
+      con = con,
+      table = ident("df_x"),
+      from = sql_render(df_y, con, lvl = 1),
       by = c("a", "b"),
       update_cols = c("c", "d"),
       returning_cols = c("a", b2 = "b")
@@ -280,6 +322,13 @@ test_that("`sql_query_upsert()` is correct", {
   )
 })
 
+test_that("row_number() with and without group_by() and arrange(): unordered defaults to Ordering by NULL (per empty_order)", {
+  mf <- lazy_frame(x = c(1:5), y = c(rep("A", 5)), con = simulate_mssql())
+  expect_snapshot(mf %>% mutate(rown = row_number()))
+  expect_snapshot(mf %>% group_by(y) %>% mutate(rown = row_number()))
+  expect_snapshot(mf %>% arrange(y) %>% mutate(rown = row_number()))
+})
+
 # Live database -----------------------------------------------------------
 
 test_that("can copy_to() and compute() with temporary tables (#272)", {
@@ -338,12 +387,10 @@ test_that("can insert", {
   con <- src_test("mssql")
 
   df_x <- tibble(a = 1L, b = 11L, c = 1L, d = "a")
-  x <- copy_to(con, df_x, "df_x", temporary = TRUE, overwrite = TRUE)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_x")))
+  x <- local_db_table(con, df_x, "df_x")
   df_y <- tibble(a = 2:3, b = c(12L, 13L), c = -(2:3), d = c("y", "z"))
-  y <- copy_to(con, df_y, "df_y", temporary = TRUE, overwrite = TRUE) %>%
+  y <- local_db_table(con, df_y, "df_y", temporary = TRUE, overwrite = TRUE) %>%
     mutate(c = c + 1)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_y")))
 
   expect_equal(
     rows_insert(
@@ -366,12 +413,10 @@ test_that("can insert with returning", {
   con <- src_test("mssql")
 
   df_x <- tibble(a = 1L, b = 11L, c = 1L, d = "a")
-  x <- copy_to(con, df_x, "df_x", temporary = TRUE, overwrite = TRUE)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_x")))
+  x <- local_db_table(con, df_x, "df_x")
   df_y <- tibble(a = 2:3, b = c(12L, 13L), c = -(2:3), d = c("y", "z"))
-  y <- copy_to(con, df_y, "df_y", temporary = TRUE, overwrite = TRUE) %>%
+  y <- local_db_table(con, df_y, "df_y") %>%
     mutate(c = c + 1)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_y")))
 
   expect_equal(
     rows_insert(
@@ -395,12 +440,10 @@ test_that("can append", {
   con <- src_test("mssql")
 
   df_x <- tibble(a = 1L, b = 11L, c = 1L, d = "a")
-  x <- copy_to(con, df_x, "df_x", temporary = TRUE, overwrite = TRUE)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_x")))
+  x <- local_db_table(con, df_x, "df_x")
   df_y <- tibble(a = 1:3, b = 11:13, c = -(2:4), d = c("y", "z", "w"))
-  y <- copy_to(con, df_y, "df_y", temporary = TRUE, overwrite = TRUE) %>%
+  y <- local_db_table(con, df_y, "df_y") %>%
     mutate(c = c + 1)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_y")))
 
   expect_equal(
     rows_append(
@@ -421,12 +464,10 @@ test_that("can append with returning", {
   con <- src_test("mssql")
 
   df_x <- tibble(a = 1L, b = 11L, c = 1L, d = "a")
-  x <- copy_to(con, df_x, "df_x", temporary = TRUE, overwrite = TRUE)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_x")))
+  x <- local_db_table(con, df_x, "df_x")
   df_y <- tibble(a = 1:3, b = 11:13, c = -(2:4), d = c("y", "z", "w"))
-  y <- copy_to(con, df_y, "df_y", temporary = TRUE, overwrite = TRUE) %>%
+  y <- local_db_table(con, df_y, "df_y") %>%
     mutate(c = c + 1)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_y")))
 
   expect_equal(
     rows_append(
@@ -448,12 +489,10 @@ test_that("can update", {
   con <- src_test("mssql")
 
   df_x <- tibble(a = 1:3, b = 11:13, c = 1:3, d = c("a", "b", "c"))
-  x <- copy_to(con, df_x, "df_x", temporary = TRUE, overwrite = TRUE)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_x")))
+  x <- local_db_table(con, df_x, "df_x")
   df_y <- tibble(a = 2:3, b = c(12L, 13L), c = -(2:3), d = c("y", "z"))
-  y <- copy_to(con, df_y, "df_y", temporary = TRUE, overwrite = TRUE) %>%
+  y <- local_db_table(con, df_y, "df_y") %>%
     mutate(c = c + 1)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_y")))
 
   expect_equal(
     rows_update(
@@ -476,12 +515,10 @@ test_that("can update with returning", {
   con <- src_test("mssql")
 
   df_x <- tibble(a = 1:3, b = 11:13, c = 1:3, d = c("a", "b", "c"))
-  x <- copy_to(con, df_x, "df_x", temporary = TRUE, overwrite = TRUE)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_x")))
+  x <- local_db_table(con, df_x, "df_x")
   df_y <- tibble(a = 2:3, b = c(12L, 13L), c = -(2:3), d = c("y", "z"))
-  y <- copy_to(con, df_y, "df_y", temporary = TRUE, overwrite = TRUE) %>%
+  y <- local_db_table(con, df_y, "df_y") %>%
     mutate(c = c + 1)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_y")))
 
   expect_equal(
     rows_update(
@@ -505,12 +542,10 @@ test_that("can upsert", {
   con <- src_test("mssql")
 
   df_x <- tibble(a = 1:2, b = 11:12, c = 1:2, d = c("a", "b"))
-  x <- copy_to(con, df_x, "df_x", temporary = TRUE, overwrite = TRUE)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_x")))
+  x <- local_db_table(con, df_x, "df_x")
   df_y <- tibble(a = 2:3, b = c(12L, 13L), c = -(2:3), d = c("y", "z"))
-  y <- copy_to(con, df_y, "df_y", temporary = TRUE, overwrite = TRUE) %>%
+  y <- local_db_table(con, df_y, "df_y") %>%
     mutate(c = c + 1)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_y")))
 
   expect_equal(
     rows_upsert(
@@ -532,12 +567,10 @@ test_that("can upsert with returning", {
   con <- src_test("mssql")
 
   df_x <- tibble(a = 1:2, b = 11:12, c = 1:2, d = c("a", "b"))
-  x <- copy_to(con, df_x, "df_x", temporary = TRUE, overwrite = TRUE)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_x")))
+  x <- local_db_table(con, df_x, "df_x")
   df_y <- tibble(a = 2:3, b = c(12L, 13L), c = -(2:3), d = c("y", "z"))
-  y <- copy_to(con, df_y, "df_y", temporary = TRUE, overwrite = TRUE) %>%
+  y <- local_db_table(con, df_y, "df_y") %>%
     mutate(c = c + 1)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_y")))
 
   expect_equal(
     rows_upsert(
@@ -561,11 +594,9 @@ test_that("can delete", {
   con <- src_test("mssql")
 
   df_x <- tibble(a = 1:3, b = 11:13, c = 1:3, d = c("a", "b", "c"))
-  x <- copy_to(con, df_x, "df_x", temporary = TRUE, overwrite = TRUE)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_x")))
+  x <- local_db_table(con, df_x, "df_x")
   df_y <- tibble(a = 2:3, b = c(12L, 13L))
-  y <- copy_to(con, df_y, "df_y", temporary = TRUE, overwrite = TRUE)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_y")))
+  y <- local_db_table(con, df_y, "df_y")
 
   expect_equal(
     rows_delete(
@@ -588,11 +619,9 @@ test_that("can delete with returning", {
   con <- src_test("mssql")
 
   df_x <- tibble(a = 1:3, b = 11:13, c = 1:3, d = c("a", "b", "c"))
-  x <- copy_to(con, df_x, "df_x", temporary = TRUE, overwrite = TRUE)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_x")))
+  x <- local_db_table(con, df_x, "df_x")
   df_y <- tibble(a = 2:3, b = c(12L, 13L))
-  y <- copy_to(con, df_y, "df_y", temporary = TRUE, overwrite = TRUE)
-  withr::defer(DBI::dbRemoveTable(con, DBI::SQL("#df_y")))
+  y <- local_db_table(con, df_y, "df_y")
 
   expect_equal(
     rows_delete(
