@@ -4,13 +4,13 @@ test_that("adds src class", {
 })
 
 test_that("argument src is deprecated", {
-  expect_snapshot(dummy <- tbl_lazy(mtcars, src = simulate_sqlite()))
+  expect_snapshot(error = TRUE, tbl_lazy(mtcars, src = simulate_sqlite()))
 })
 
 test_that("cannot convert tbl_lazy to data.frame", {
   expect_snapshot(
     error = TRUE,
-    as.data.frame(tbl_lazy(mtcars, src = simulate_sqlite()))
+    as.data.frame(tbl_lazy(mtcars, con = simulate_sqlite()))
   )
 })
 
@@ -38,5 +38,16 @@ test_that("support colwise variants", {
 
 test_that("base source of tbl_lazy is always 'df'", {
   out <- lazy_frame(x = 1, y = 5) %>% sql_build()
-  expect_equal(out, ident("df"))
+  expect_equal(out, base_query(ident("df")))
+})
+
+test_that("names() inform that they aren't meant to be used", {
+  expect_snapshot(names(lazy_frame(x = 1)))
+})
+
+test_that("$ aborts when not used with src or lazy_query", {
+  lf <- lazy_frame(x = 1)
+  expect_no_error(lf$src)
+  expect_no_error(lf$lazy_query)
+  expect_snapshot_error(lf$x)
 })
